/* This file contains definitions of the basic
 * ANSI color codes for use in printf statements.
 * Prepared by Mark Veinot.
 */

#ifndef _COLORS_H
#define _COLORS_H

#define BLACK		"\x1B[0;30m"
#define DARKRED		"\x1B[0;31m"
#define DARKGREEN	"\x1B[0;32m"
#define ORANGE		"\x1B[0;33m"
#define DARKBLUE	"\x1B[0;34m"
#define VIOLET		"\x1B[0;35m"
#define DARKCYAN	"\x1B[0;36m"
#define GRAY		"\x1B[0;37m"
#define GREY            "\x1B[0;37m"
#define DARKGRAY	"\x1B[1;30m"
#define DARKGREY	"\x1B[1;30m"
#define RED		"\x1B[1;31m"
#define GREEN		"\x1B[1;32m"
#define YELLOW		"\x1B[1;33m"
#define BLUE		"\x1B[1;34m"
#define MAGENTA		"\x1B[1;35m"
#define CYAN		"\x1B[1;36m"
#define WHITE		"\x1B[1;37m"
#define NORMAL		"\x1B[0m"

#endif	/* _COLORS_H */

