/* This file defines the basic ANSI colors to
 * empty strings. This is used when NOCOLOR is defined
 */

#ifndef _NOCOLORS_H
#define _NOCOLORS_H

#define BLACK		""
#define DARKRED		""
#define DARKGREEN	""
#define ORANGE		""
#define DARKBLUE	""
#define VIOLET		""
#define DARKCYAN	""
#define GRAY		""
#define GREY            ""
#define DARKGRAY	""
#define DARKGREY	""
#define RED		""
#define GREEN		""
#define YELLOW		""
#define BLUE		""
#define MAGENTA		""
#define CYAN		""
#define WHITE		""
#define NORMAL		""

#endif	/* _NOCOLORS_H */

