/* ibeat.h
 * ibeat calculation routines
 * by iain truskett
 */

#ifndef _IBEAT_H
#define _IBEAT_H

#include <time.h>

/* iain truskett's little ibeat calculator */
int time_ibeats();

#endif	/* _IBEAT_H */

